==> connect-sequence-0.9.0.tgz: package/tests/ConnectSequence.spec.js <==
'use strict'

var path = require('path')
var chai = require('chai')
var ConnectSequence = require(path.resolve('./lib/ConnectSequence'))
var MissingArgumentError = require(path.resolve('./lib/errors/MissingArgumentError'))

var describe = global.describe
var beforeEach = global.beforeEach
var it = global.it
var expect = chai.expect

process.env.NODE_ENV = 'test'

describe('ConnectSequence', function () {
  it('should be a function', function () {
    expect(ConnectSequence).to.be.a('function')
  })

  it('should have a "run" property', function () {
    expect(ConnectSequence).to.have.property('run')
  })

  it('should have a prototype', function () {
    expect(ConnectSequence).to.have.property('prototype')
  })

  describe('.run() [deprecated]', function () {
    it('should be a function', function () {
      expect(ConnectSequence.run).to.be.a('function')
    })

    it('should throw an Error if one argument is missing', function () {
      var initMid = function (req, res) {}
      var noopMid = function (req, res, next) { next() }
      var run0 = function () {
        ConnectSequence.run()
      }
      var run1 = function () {
        ConnectSequence.run({})
      }
      var run2 = function () {
        ConnectSequence.run({}, {})
      }
      var run3 = function () {
        ConnectSequence.run({}, {}, function (next) { next() })
      }
      var run4 = function () {
        ConnectSequence.run({}, {}, noopMid)
      }
      var run5 = function () {
        ConnectSequence.run({}, {}, initMid, [noopMid, noopMid, noopMid])
      }
      expect(run0).to.throw(Error)
      expect(run1).to.throw(Error)
      expect(run2).to.throw(Error)
      expect(run3).to.throw(Error)
      expect(run4).to.throw(Error)
      expect(run5).to.not.throw(Error)
    })

    it('should run the initial next middleware at last', function () {
      var _req = {}
      var _res = {}
      var _next = function (req, res) {
        req.output = 'initialNext'
        expect(req.output).to.equal('initialNext')
      }
      var first = function (req, res, next) {
        req.output = 'first'
      }
      var second = first
      var third = first
      var fourth = first
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run all the middlewares in the passed array of middlewares', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids).to.contain('initial')
        expect(req.ids).to.contain('first')
        expect(req.ids).to.contain('second')
        expect(req.ids).to.contain('third')
        expect(req.ids).to.contain('fourth')
      }
      var first = function (req, res, next) {
        req.ids.push('first')
      }
      var second = function (req, res, next) {
        req.ids.push('second')
      }
      var third = function (req, res, next) {
        req.ids.push('third')
      }
      var fourth = function (req, res, next) {
        req.ids.push('fourth')
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run all the middlewares in the same order than the given array', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        req.ids.push('last')
        expect(req.ids.join()).to.equal('first,second,third,fourth,last')
      }
      var first = function (req, res, next) {
        req.ids.push('first')
      }
      var second = function (req, res, next) {
        req.ids.push('second')
      }
      var third = function (req, res, next) {
        req.ids.push('third')
      }
      var fourth = function (req, res, next) {
        req.ids.push('fourth')
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run each middleware as a callback of the previous', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        if (req && req.ids) {
          req.ids.push('last')
          expect(req.ids.join()).to.equal('first,second,third,fourth,last')
        }
      }
      var first = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('first')
          next()
        }, 150)
      }
      var second = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('second')
          next()
        }, 50)
      }
      var third = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('third')
          next()
        }, 100)
      }
      var fourth = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('fourth')
          next()
        }, 150)
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })
  })

  describe('.prototype', function () {
    it('should be an object', function () {
      expect(ConnectSequence.prototype).to.be.a('object')
    })

    it('should have a "append" method', function () {
      expect(ConnectSequence.prototype).to.have.property('append')
      expect(ConnectSequence.prototype.append).to.be.a('function')
    })

    it('should have a "appendList" method', function () {
      expect(ConnectSequence.prototype).to.have.property('appendList')
      expect(ConnectSequence.prototype.appendList).to.be.a('function')
    })

    it('should have a "run" method', function () {
      expect(ConnectSequence.prototype).to.have.property('run')
      expect(ConnectSequence.prototype.appendList).to.be.a('function')
    })

    describe('.constructor', function () {
      it('should be an object of type "ConnectSequence"', function () {
        var seq = new ConnectSequence()
        expect(seq).to.be.a('object')
        expect(seq).to.be.an.instanceof(ConnectSequence)
      })

      it('should init the "middleware" instance property as an empty Array', function () {
        var seq = new ConnectSequence()
        expect(seq).to.has.a.property('middlewares')
        expect(seq.middlewares).to.be.a('array')
        expect(seq.middlewares.length).to.be.equal(0)
      })
    })
  })

  describe('#append()', function () {
    it('should augments the length of the middlewares array by the number of given middlewares', function () {
      var seq = new ConnectSequence()
      var mid = function (req, res, next) { next() }
      seq.append(mid)
      expect(seq.middlewares.length).to.equal(1)
      seq.append(mid)
      expect(seq.middlewares.length).to.equal(2)
      seq.append(mid, mid)
      expect(seq.middlewares.length).to.equal(4)
      seq.append()
      expect(seq.middlewares.length).to.equal(4)
      seq.append(mid, mid, mid)
      expect(seq.middlewares.length).to.equal(7)
    })

    it('should keep the same order of the given middlewares', function () {
      var seq = new ConnectSequence()
      var mid0 = function (req, res, next) { next() }
      var mid1 = function (req, res, next) { next() }
      var mid2 = function (req, res, next) { next() }
      var mid3 = function (req, res, next) { next() }
      var mid4 = function (req, res, next) { next() }
      var mid5 = function (req, res, next) { next() }
      var mid6 = function (req, res, next) { next() }
      seq.append(mid0)
      seq.append(mid1, mid2)
      seq.append()
      seq.append(mid3, mid4, mid5)
      seq.append(mid6)
      var mids = [mid0, mid1, mid2, mid3, mid4, mid5, mid6]
      for (var i = 0; i < mids.length; i++) {
        var mid = mids[i]
        expect(seq.middlewares[i]).to.equal(mid)
      }
    })

    it('should throw a TypeError and reject all middlewares if one is not a function', function () {
      var seq = new ConnectSequence()
      var mid1 = function (req, res, next) { next() }
      var mid2 = 'not a function'
      var mid3 = function (req, res, next) { next() }
      var cantAppend = function () {
        seq.append(mid1, mid2, mid3)
      }
      var shouldAppend = function () {
        seq.append(mid1, mid3)
      }
      expect(cantAppend).to.throw(TypeError)
      expect(seq.middlewares.length).to.be.equal(0)
      expect(shouldAppend).to.not.throw(Error)
      expect(seq.middlewares.length).to.be.equal(2)
    })
  })

  describe('#appendList()', function () {
    it('should throw TypeError if the first argument is not an array', function () {
      var funcs = [
        function () {
          var seq = new ConnectSequence()
          seq.appendList('not an array')
        },
        function () {
          var seq = new ConnectSequence()
          seq.appendList({foo: 'not an array'})
        },
        function () {
          var seq = new ConnectSequence()
          seq.appendList(function () { return 'not an array' })
        }
      ]
      for (var i = 0; i < funcs.length; i++) {
        expect(funcs[i]).to.throw(TypeError)
      }
    })

    it('should augments the length of the middlewares array by the number of given middlewares', function () {
      var seq = new ConnectSequence()
      var mid = function (req, res, next) { next() }
      seq.appendList([mid])
      expect(seq.middlewares.length).to.equal(1)
      seq.appendList([mid])
      expect(seq.middlewares.length).to.equal(2)
      seq.appendList([mid, mid])
      expect(seq.middlewares.length).to.equal(4)
      seq.appendList([])
      expect(seq.middlewares.length).to.equal(4)
      seq.appendList([mid, mid, mid])
      expect(seq.middlewares.length).to.equal(7)
    })

    it('should keep the same order of the given middlewares', function () {
      var seq = new ConnectSequence()
      var mid0 = function (req, res, next) { next() }
      var mid1 = function (req, res, next) { next() }
      var mid2 = function (req, res, next) { next() }
      var mid3 = function (req, res, next) { next() }
      var mid4 = function (req, res, next) { next() }
      var mid5 = function (req, res, next) { next() }
      var mid6 = function (req, res, next) { next() }
      seq.appendList([mid0])
      seq.appendList([mid1, mid2])
      seq.appendList([])
      seq.appendList([mid3, mid4, mid5])
      seq.appendList([mid6])
      var mids = [mid0, mid1, mid2, mid3, mid4, mid5, mid6]
      for (var i = 0; i < mids.length; i++) {
        var mid = mids[i]
        expect(seq.middlewares[i]).to.equal(mid)
      }
    })

    it('should throw a TypeError and reject all middlewares if one is not a function', function () {
      var seq = new ConnectSequence()
      var mid1 = function (req, res, next) { next() }
      var mid2 = 'not a function'
      var mid3 = function (req, res, next) { next() }
      var cantAppend = function () {
        seq.appendList([mid1, mid2, mid3])
      }
      var shouldAppend = function () {
        seq.appendList([mid1, mid3])
      }
      expect(cantAppend).to.throw(TypeError)
      expect(seq.middlewares.length).to.be.equal(0)
      expect(shouldAppend).to.not.throw(Error)
      expect(seq.middlewares.length).to.be.equal(2)
    })
  })

  describe('#run()', function () {
    var seq, req, res, next
    var mid0, mid1, mid2, mid3
    var func0, func1, func2

    beforeEach(function () {
      seq = new ConnectSequence()
      req = {}
      res = {}
      mid0 = function mid0 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid0')
        next()
      }
      mid1 = function mid1 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid1')
        next()
      }
      mid2 = function mid2 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid2')
        next()
      }
      mid3 = function mid3 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid3')
        next()
      }
    })

    it('should be a function', function () {
      expect(seq.run).to.be.a('function')
    })

    it('should throw MissingArgumentError if called with lower than 3 arguments', function () {
      func0 = function () {
        seq.appendList([mid0, mid1, mid2, mid3])
        seq.run(req, res)
      }
      expect(func0).to.throw(MissingArgumentError)
    })

    it('should throw TypeError if the given arguments have a bad type', function () {
      next = function (req, res) { return true }
      seq.append(mid0, mid1, mid2, mid3)

      func0 = function () { seq.run('not an object', {}, next) }
      func1 = function () { seq.run({}, 'not an object', next) }
      func2 = function () { seq.run({}, {}, 'not a function') }

      var funcs = [func0, func1, func2]
      for (var i = 0; i < funcs.length; i++) {
        expect(funcs[i]).to.throw(TypeError)
      }
    })

    it('should run the initial next middleware at last', function (done) {
      next = function (req, res) {
        req.ids = 'initialNext'
        done()
      }
      seq.append(mid0, mid0, mid0, mid0)
      seq.run(req, res, next)
      expect(req.ids).to.be.a(String)
      expect(req.ids).to.equal('initialNext')
    })

    it('should run all the middlewares in the passed array of middlewares', function (done) {
      next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids).to.contain('initial')
        expect(req.ids).to.contain('mid0')
        expect(req.ids).to.contain('mid1')
        expect(req.ids).to.contain('mid2')
        expect(req.ids).to.contain('mid3')
        setTimeout(done, 20)
      }
      seq.append(mid0, mid1, mid2, mid3)
      seq.run(req, res, next)
    })

    it('should run all the middlewares in the same order than the given array', function (done) {
      next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids.join()).to.equal('mid0,mid1,mid2,mid3,initial')
        setTimeout(done, 20)
      }
      seq.append(mid0, mid1, mid2, mid3)
      seq.run(req, res, next)
    })

    it('should run each middleware as a callback of the previous', function (done) {
      this.slow(500)
      next = function (req, res) {
        if (req && req.ids) {
          req.ids.push('initial')
          expect(req.ids.join()).to.equal('mid0,mid1,mid2,mid3,initial')
          done()
        }
      }
      var _mid0 = function (req, res, next) {
        setTimeout(mid0.bind(null, req, res, next), 150)
      }
      var _mid1 = function (req, res, next) {
        setTimeout(mid1.bind(null, req, res, next), 50)
      }
      var _mid2 = function (req, res, next) {
        setTimeout(mid2.bind(null, req, res, next), 100)
      }
      var _mid3 = function (req, res, next) {
        setTimeout(mid3.bind(null, req, res, next), 150)
      }
      seq.append(_mid0, _mid1, _mid2, _mid3)
      seq.run(req, res, next)
    })
  })
})

function ensureReqIdsDefined (req) {
  if (!req.ids) {
    req.ids = []
  }
}

==> connect-sequence-1.3.0.tgz: package/tests/ConnectSequence.spec.js <==
'use strict'

var path = require('path')
var chai = require('chai')
var ConnectSequence = require(path.resolve('./lib/ConnectSequence'))
var MissingArgumentError = require(path.resolve('./lib/errors/MissingArgumentError'))

var describe = global.describe
var beforeEach = global.beforeEach
var it = global.it
var expect = chai.expect

describe('ConnectSequence', function () {
  var seq, req, res, next, mid

  beforeEach(function () {
    req = { foo: 'bar' }
    res = {}
    next = function (req, res) { res.foo = req.foo }
    mid = function (req, res, next) { req.foo = 'baz'; next() }
    seq = new ConnectSequence(req, res, next)
  })

  it('should be a function', function () {
    expect(ConnectSequence).to.be.a('function')
  })

  it('should have a "run" property', function () {
    expect(ConnectSequence).to.have.property('run')
  })

  it('should have a prototype', function () {
    expect(ConnectSequence).to.have.property('prototype')
  })

  describe('.run() [deprecated]', function () {
    it('should be a function', function () {
      expect(ConnectSequence.run).to.be.a('function')
    })

    it('should throw an Error if one argument is missing', function () {
      var initMid = function (req, res) {}
      var noopMid = function (req, res, next) { next() }
      var run0 = function () {
        ConnectSequence.run()
      }
      var run1 = function () {
        ConnectSequence.run({})
      }
      var run2 = function () {
        ConnectSequence.run({}, {})
      }
      var run3 = function () {
        ConnectSequence.run({}, {}, function (next) { next() })
      }
      var run4 = function () {
        ConnectSequence.run({}, {}, noopMid)
      }
      var run5 = function () {
        ConnectSequence.run({}, {}, initMid, [noopMid, noopMid, noopMid])
      }
      expect(run0).to.throw(Error)
      expect(run1).to.throw(Error)
      expect(run2).to.throw(Error)
      expect(run3).to.throw(Error)
      expect(run4).to.throw(Error)
      expect(run5).to.not.throw(Error)
    })

    it('should run the initial next middleware at last', function () {
      var _req = {}
      var _res = {}
      var _next = function (req, res) {
        req.output = 'initialNext'
        expect(req.output).to.equal('initialNext')
      }
      var first = function (req, res, next) {
        req.output = 'first'
      }
      var second = first
      var third = first
      var fourth = first
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run all the middlewares in the passed array of middlewares', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids).to.contain('initial')
        expect(req.ids).to.contain('first')
        expect(req.ids).to.contain('second')
        expect(req.ids).to.contain('third')
        expect(req.ids).to.contain('fourth')
      }
      var first = function (req, res, next) {
        req.ids.push('first')
      }
      var second = function (req, res, next) {
        req.ids.push('second')
      }
      var third = function (req, res, next) {
        req.ids.push('third')
      }
      var fourth = function (req, res, next) {
        req.ids.push('fourth')
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run all the middlewares in the same order than the given array', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        req.ids.push('last')
        expect(req.ids.join()).to.equal('first,second,third,fourth,last')
      }
      var first = function (req, res, next) {
        req.ids.push('first')
      }
      var second = function (req, res, next) {
        req.ids.push('second')
      }
      var third = function (req, res, next) {
        req.ids.push('third')
      }
      var fourth = function (req, res, next) {
        req.ids.push('fourth')
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })

    it('should run each middleware as a callback of the previous', function () {
      var _req = {
        ids: []
      }
      var _res = {}
      var _next = function (req, res) {
        if (req && req.ids) {
          req.ids.push('last')
          expect(req.ids.join()).to.equal('first,second,third,fourth,last')
        }
      }
      var first = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('first')
          next()
        }, 150)
      }
      var second = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('second')
          next()
        }, 50)
      }
      var third = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('third')
          next()
        }, 100)
      }
      var fourth = function (req, res, next) {
        setTimeout(function () {
          req.ids.push('fourth')
          next()
        }, 150)
      }
      var mids = [first, second, third, fourth]

      ConnectSequence.run(_req, _res, _next, mids)
    })
  })

  describe('.prototype', function () {
    it('should be an object', function () {
      expect(ConnectSequence.prototype).to.be.a('object')
    })

    it('should have a "append" method', function () {
      expect(ConnectSequence.prototype).to.have.property('append')
      expect(ConnectSequence.prototype.append).to.be.a('function')
    })

    it('should have a "appendList" method', function () {
      expect(ConnectSequence.prototype).to.have.property('appendList')
      expect(ConnectSequence.prototype.appendList).to.be.a('function')
    })

    it('should have a "appendIf" method', function () {
      expect(ConnectSequence.prototype).to.have.property('appendIf')
      expect(ConnectSequence.prototype.appendIf).to.be.a('function')
    })

    it('should have a "run" method', function () {
      expect(ConnectSequence.prototype).to.have.property('run')
      expect(ConnectSequence.prototype.appendList).to.be.a('function')
    })

    describe('.constructor()', function () {
      it('should be an object of type `ConnectSequence`', function () {
        seq = new ConnectSequence(req, res, next)
        expect(seq).to.be.a('object')
        expect(seq).to.be.an.instanceof(ConnectSequence)
      })

      it('should init the `middlewares` instance property as an empty Array', function () {
        seq = new ConnectSequence(req, res, next)
        expect(seq).to.has.a.property('middlewares')
        expect(seq.middlewares).to.be.a('array')
        expect(seq.middlewares.length).to.be.equal(0)
      })

      it('should throw `MissingArgumentError` if called with lower than 3 arguments', function () {
        expect(function () {
          seq = new ConnectSequence({})
        }).to.throw(MissingArgumentError)

        expect(function () {
          seq = new ConnectSequence({}, {})
        }).to.throw(MissingArgumentError)
      })

      it('should throw `TypeError` if the first argument is not an object', function () {
        expect(function () {
          seq = new ConnectSequence('not an object', {}, function (req, res) { return true })
        }).to.throw(TypeError)
      })

      it('should throw `TypeError` if the second argument is not an object', function () {
        expect(function () {
          seq = new ConnectSequence({}, 'not an object', function (req, res) { return true })
        }).to.throw(TypeError)
      })

      it('should throw `TypeError` if the third argument is not a function', function () {
        expect(function () {
          seq = new ConnectSequence({}, {}, 'not a function')
        }).to.throw(TypeError)
      })

      it('should init the `req` object', function () {
        seq = new ConnectSequence(req, res, next)
        expect(seq.req).to.equal(req)
      })

      it('should init the `res` object', function () {
        seq = new ConnectSequence(req, res, next)
        expect(seq.res).to.equal(res)
      })

      it('should init the `next` function', function () {
        seq = new ConnectSequence(req, res, next)
        expect(seq.next).to.equal(next)
      })
    })
  })

  describe('#append()', function () {
    it('should augments the length of the middlewares array by the number of given middlewares', function () {
      seq = new ConnectSequence(req, res, next)
      var mid = function (req, res, next) { next() }
      seq.append(mid)
      expect(seq.middlewares.length).to.equal(1)
      seq.append(mid)
      expect(seq.middlewares.length).to.equal(2)
      seq.append(mid, mid)
      expect(seq.middlewares.length).to.equal(4)
      seq.append()
      expect(seq.middlewares.length).to.equal(4)
      seq.append(mid, mid, mid)
      expect(seq.middlewares.length).to.equal(7)
    })

    it('should keep the same order of the given middlewares', function () {
      seq = new ConnectSequence(req, res, next)
      var mid0 = function (req, res, next) { next() }
      var mid1 = function (req, res, next) { next() }
      var mid2 = function (req, res, next) { next() }
      var mid3 = function (req, res, next) { next() }
      var mid4 = function (req, res, next) { next() }
      var mid5 = function (req, res, next) { next() }
      var mid6 = function (req, res, next) { next() }
      seq.append(mid0)
      seq.append(mid1, mid2)
      seq.append()
      seq.append(mid3, mid4, mid5)
      seq.append(mid6)
      var mids = [mid0, mid1, mid2, mid3, mid4, mid5, mid6]
      for (var i = 0; i < mids.length; i++) {
        var mid = mids[i]
        expect(seq.middlewares[i]).to.equal(mid)
      }
    })

    it('should throw a TypeError and reject all middlewares if one is not a function', function () {
      seq = new ConnectSequence(req, res, next)
      var mid1 = function (req, res, next) { next() }
      var mid2 = 'not a function'
      var mid3 = function (req, res, next) { next() }
      var cantAppend = function () {
        seq.append(mid1, mid2, mid3)
      }
      var shouldAppend = function () {
        seq.append(mid1, mid3)
      }
      expect(cantAppend).to.throw(TypeError)
      expect(seq.middlewares.length).to.be.equal(0)
      expect(shouldAppend).to.not.throw(Error)
      expect(seq.middlewares.length).to.be.equal(2)
    })
  })

  describe('#appendList()', function () {
    it('should throw TypeError if the first argument is not an array', function () {
      var funcs = [
        function () {
          seq = new ConnectSequence(req, res, next)
          seq.appendList('not an array')
        },
        function () {
          seq = new ConnectSequence(req, res, next)
          seq.appendList({foo: 'not an array'})
        },
        function () {
          seq = new ConnectSequence(req, res, next)
          seq.appendList(function () { return 'not an array' })
        }
      ]
      for (var i = 0; i < funcs.length; i++) {
        expect(funcs[i]).to.throw(TypeError)
      }
    })

    it('should augments the length of the middlewares array by the number of given middlewares', function () {
      seq = new ConnectSequence(req, res, next)
      var mid = function (req, res, next) { next() }
      seq.appendList([mid])
      expect(seq.middlewares.length).to.equal(1)
      seq.appendList([mid])
      expect(seq.middlewares.length).to.equal(2)
      seq.appendList([mid, mid])
      expect(seq.middlewares.length).to.equal(4)
      seq.appendList([])
      expect(seq.middlewares.length).to.equal(4)
      seq.appendList([mid, mid, mid])
      expect(seq.middlewares.length).to.equal(7)
    })

    it('should keep the same order of the given middlewares', function () {
      seq = new ConnectSequence(req, res, next)
      var mid0 = function (req, res, next) { next() }
      var mid1 = function (req, res, next) { next() }
      var mid2 = function (req, res, next) { next() }
      var mid3 = function (req, res, next) { next() }
      var mid4 = function (req, res, next) { next() }
      var mid5 = function (req, res, next) { next() }
      var mid6 = function (req, res, next) { next() }
      seq.appendList([mid0])
      seq.appendList([mid1, mid2])
      seq.appendList([])
      seq.appendList([mid3, mid4, mid5])
      seq.appendList([mid6])
      var mids = [mid0, mid1, mid2, mid3, mid4, mid5, mid6]
      for (var i = 0; i < mids.length; i++) {
        var mid = mids[i]
        expect(seq.middlewares[i]).to.equal(mid)
      }
    })

    it('should throw a TypeError and reject all middlewares if one is not a function', function () {
      seq = new ConnectSequence(req, res, next)
      var mid1 = function (req, res, next) { next() }
      var mid2 = 'not a function'
      var mid3 = function (req, res, next) { next() }
      var cantAppend = function () {
        seq.appendList([mid1, mid2, mid3])
      }
      var shouldAppend = function () {
        seq.appendList([mid1, mid3])
      }
      expect(cantAppend).to.throw(TypeError)
      expect(seq.middlewares.length).to.be.equal(0)
      expect(shouldAppend).to.not.throw(Error)
      expect(seq.middlewares.length).to.be.equal(2)
    })
  })

  describe('#appendIf()', function () {
    var filter

    beforeEach(function () {
      filter = function (req) { return !!req.foo }
    })

    it('should throw `MissingArgumentError` if called with lower than 2 arguments', function () {
      expect(function () {
        seq.appendIf()
      }).to.throw(MissingArgumentError)

      expect(function () {
        seq.appendIf(filter)
      }).to.throw(MissingArgumentError)

      expect(function () {
        seq.appendIf(filter, mid)
      }).to.not.throw(MissingArgumentError)
    })

    it('should throw `TypeError` if the first arguments is not a function', function () {
      expect(function () {
        seq.appendIf('not a function', mid)
      }).to.throw(TypeError)

      expect(function () {
        seq.appendIf(filter, mid)
      }).to.not.throw(Error)
    })

    it('should throw `TypeError` if second first arguments is not a function', function () {
      expect(function () {
        seq.appendIf(filter, 'not a function')
      }).to.throw(TypeError)

      expect(function () {
        seq.appendIf(filter, mid)
      }).to.not.throw(Error)
    })

    describe('when the previous condition on `req` is `true`', function () {
      it('should run the given middleware', function (done) {
        next = function (req, res) {
          expect(res.foo).to.equal('bar')
          done()
        }
        seq = new ConnectSequence(req, res, next)
        seq.appendIf(function (req) {
          return req.foo === 'bar'
        }, function (req, res, next) {
          res.foo = req.foo // 'bar'
          next()
        })
        seq.run()
      })
    })

    describe('when the previous condition on `req` is `false`', function () {
      it('should not run the given middleware', function (done) {
        next = function (req, res) {
          expect(res.foo).to.not.equal('bar')
          expect(res.foo).to.be.undefined
          done()
        }
        seq = new ConnectSequence(req, res, next)
        seq.appendIf(function (req) {
          return !req.foo
        }, function (req, res, next) {
          res.foo = req.foo // 'bar'
          next()
        })
        seq.run()
      })
    })
  })

  describe('#run()', function () {
    var mid0, mid1, mid2, mid3

    beforeEach(function () {
      seq = new ConnectSequence(req, res, next)
      mid0 = function mid0 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid0')
        next()
      }
      mid1 = function mid1 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid1')
        next()
      }
      mid2 = function mid2 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid2')
        next()
      }
      mid3 = function mid3 (req, res, next) {
        ensureReqIdsDefined(req)
        req.ids.push('mid3')
        next()
      }
    })

    it('should be a function', function () {
      expect(seq.run).to.be.a('function')
    })

    it('should run the initial next middleware at last', function (done) {
      next = function (req, res) {
        req.ids = 'initialNext'
        done()
      }
      seq = new ConnectSequence(req, res, next)
      seq.append(mid0, mid0, mid0, mid0)
      seq.run()
      expect(req.ids).to.be.a(String)
      expect(req.ids).to.equal('initialNext')
    })

    it('should run all the middlewares in the passed array of middlewares', function (done) {
      next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids).to.contain('initial')
        expect(req.ids).to.contain('mid0')
        expect(req.ids).to.contain('mid1')
        expect(req.ids).to.contain('mid2')
        expect(req.ids).to.contain('mid3')
        setTimeout(done, 20)
      }
      seq = new ConnectSequence(req, res, next)
      seq.append(mid0, mid1, mid2, mid3)
      seq.run()
    })

    it('should run all the middlewares in the same order than the given array', function (done) {
      next = function (req, res) {
        req.ids.push('initial')
        expect(req.ids.join()).to.equal('mid0,mid1,mid2,mid3,initial')
        setTimeout(done, 20)
      }
      seq = new ConnectSequence(req, res, next)
      seq.append(mid0, mid1, mid2, mid3)
      seq.run()
    })

    it('should run each middleware as a callback of the previous', function (done) {
      this.slow(500)
      next = function (req, res) {
        if (req && req.ids) {
          req.ids.push('initial')
          expect(req.ids.join()).to.equal('mid0,mid1,mid2,mid3,initial')
          done()
        }
      }
      var _mid0 = function (req, res, next) {
        setTimeout(mid0.bind(null, req, res, next), 150)
      }
      var _mid1 = function (req, res, next) {
        setTimeout(mid1.bind(null, req, res, next), 50)
      }
      var _mid2 = function (req, res, next) {
        setTimeout(mid2.bind(null, req, res, next), 100)
      }
      var _mid3 = function (req, res, next) {
        setTimeout(mid3.bind(null, req, res, next), 150)
      }
      seq = new ConnectSequence(req, res, next)
      seq.append(_mid0, _mid1, _mid2, _mid3)
      seq.run(req, res, next)
    })
  })
})

function ensureReqIdsDefined (req) {
  if (!req.ids) {
    req.ids = []
  }
}
